#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <Palettes.h>
#include <console.h>
#include "wirefram.h"

#undef min
#undef max
#define min(x,y) (x<y?x:y)
#define max(x,y) (x>y?x:y)

int window_width, window_height;
double offsetx, offsety;
Rect actual_view;

static FILE *TargaFile;

static WindowPtr cWindow;
static PaletteHandle Wire_Palette;

void
ToolBoxInit()
{
        InitGraf(&thePort);
        InitFonts();
        FlushEvents(0xFFFF, 0);
        InitWindows();
        InitMenus();
        TEInit();
        InitDialogs(0L);
        InitCursor();
}

void display_init()
{
   RGBColor c;
   int index, i, j;
   int r, g, b;
   Rect re;
        
   ToolBoxInit();
        
   actual_view.top = 50;
   actual_view.left = 250;
   actual_view.right = 250 + 320;
   actual_view.bottom = 50 + 240;
   cWindow = NewCWindow(0L, &actual_view, "\pWire Frame Display",
                        FALSE, noGrowDocProc,
                        (WindowPtr)-1L, FALSE, 0L);
   actual_view.top = 0;
   actual_view.left = 0;
   actual_view.right = 319;
   actual_view.bottom = 239;

   Wire_Palette = NewPalette(256, 0L, pmTolerant, 0x0000);
   c.red = c.green = c.blue = 0;
   SetEntryColor(Wire_Palette, 0, &c);

   for (r=0;r<8;r++)
      for (g=0;g<8;g++)
         for (b=0;b<4;b++) {
            index = (r * 32 + g * 4 + b);
            c.red   = (r * 9362);
            c.green = (g * 9362);
            c.blue  = (b * 21845);
            SetEntryColor(Wire_Palette, index, &c);
            }

   SetPalette(cWindow, Wire_Palette, TRUE);
   /* Now that the palette has been created, open the window. */
   ShowWindow(cWindow);
   SetPort(cWindow);

   /* Show what the palette looks like */
   /*
   for (i=0;i<16;i++) {
      for (j=0;j<16;j++) {
         PmForeColor(i * 16 + j);
         re.left   = i * 16;
         re.right  = (i+1)*16;
         re.top    = j * 8;
         re.bottom = (j+1) * 8;
         PaintRect(&re);
         }
      }
   getchar();
   */
   return;
}

/* Initialize graphics mode. */
void
initialize_graphics()
{
   display_init();
   offsetx = (double)(actual_view.right - actual_view.left) / 2.0;
   offsety = (double)(actual_view.bottom - actual_view.top) / 2.0;
   window_width = (actual_view.right - actual_view.left) + 1;
   window_height = (actual_view.bottom - actual_view.top) + 1;
}

/* Decode 332 */
int
get_indexed_color(VECTOR color)
{
   int icolor, Red, Green, Blue;

   Red   = (int)(7.0 * color.x);
   Green = (int)(7.0 * color.y);
   Blue  = (int)(3.0 * color.z);
   icolor = (int)(Red * 32 + Green * 4 + Blue);
   /* printf("Color for <%f, %f, %f> is: %d\n",
          color.x, color.y, color.z, icolor); */
   return icolor;
}


void
putpixel(x, y, color)
   int x, y, color;
{
   Rect r;
   r.top = y;
   r.left = x;
   r.bottom = y+1;
   r.right = x+1;
   PmForeColor(color);
   PaintRect(&r);
   return;
}

void
fillpoly(left, top, right, bottom, color)
   int left, top, right, bottom, color;
{
   Rect r;
   r.top = top;
   r.left = left;
   r.bottom = bottom;
   r.right = right;
   PmForeColor(color);
   PaintRect(&r);
   return;
}

void
setcolor(color)
   int color;
{
   PmForeColor(color);
}

void
line(x0, y0, x1, y1)
   int x0, y0, x1, y1;
{
   MoveTo(x0, y0);
   LineTo(x1, y1);
}

void
closegraph()
{
}

void
clearviewport()
{
   Rect r;
   PmForeColor(0);
   r.top = actual_view.top;
   r.left = actual_view.left;
   r.bottom = actual_view.bottom;
   r.right = actual_view.right;
   PaintRect(&r);
}

void
restorecrtmode()
{
   csetmode(C_ECHO, stdin);
}

void
setgraphmode(x)
   int x;
{
   csetmode(C_RAW, stdin);
}

int
getgraphmode()
{
   return 0;
}

void
reset_pallette()
{
}

int
getch()
{
   char line[40];
   gets(&line[0]);
   return line[0];
}

void
TargaOpen(char *filename, int x, int y)
{
   FILE *file;
   unsigned char tgaheader[18];
   if ((file = fopen(filename, "wb")) == NULL) {
      printf("Failed to open Targa file: %s\n", filename);
      exit(1);
      }
   memset(tgaheader, 0, 18);
   tgaheader[2] = 2;
   tgaheader[12] = x & 0xFF;
   tgaheader[13] = (x >> 8) & 0xFF;
   tgaheader[14] = y & 0xFF;
   tgaheader[15] = (y >> 8) & 0xFF;
   tgaheader[16] = 24;
   tgaheader[17] = 0x20;
   fwrite(tgaheader, 18, 1, file);
   TargaFile = file;
}

void
TargaWrite()
{
   int i, offset;
   extern unsigned long Total_Map_Buffers, Total_Elements_Per_Buffer;
   extern Screen_Map_Type Screen_Map[64];
   for (i=0;i<Total_Map_Buffers;i++)
      for (offset=0;offset<Total_Elements_Per_Buffer;offset++) {
         fputc(Screen_Map[i][offset].blue, TargaFile);
         fputc(Screen_Map[i][offset].green, TargaFile);
         fputc(Screen_Map[i][offset].red, TargaFile);
         }
}

void
TargaClose()
{
   fclose(TargaFile);
}
